--- ALU, PSR and Akku for the DHBW MCT controller (VHDL)

use work.MCT_Pack_uP.all;
library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity ALU_PSR_Akku is
	port ( OPCODE : in OPTYPE; K, DS_DO : in D_Type; Clk, ACC_EN : in std_logic; 
	       DS_DI: out D_Type; CFlag, ZFlag, NFlag: out std_logic :='0' );
end ALU_PSR_Akku;

architecture RTL of ALU_PSR_Akku is
signal Zero : D_Type := (others => '0');
signal Akku, RdOut : D_Type := (others => '0');

begin

  process (OPCODE, K, DS_DO, Akku) 
    variable RsVar, RdVar, RdoutVar : D_Type;
    variable RVar : signed(D_Width downto 0); -- 9 bits wide
    begin
      RsVar := DS_DO; -- copy Rs from data memory  
      RdVar := Akku;  -- copy Rd from Akku
      RdoutVar := Zero; -- avoids latches in synthesis
      
      case OPCODE is
         when LSL => CFlag <= RdVar(D_Width-1); -- msb
                     RdOutVar(D_Width-1 downto 1) := RdVar(D_Width-2 downto 0);
                     RdOutVar(0):='0';          -- lsb
         when LSR => CFlag <= Akku(0);          -- lsb
                     RdOutVar(D_Width-2 downto 0) := Akku(D_Width-1 downto 1);
                     RdOutVar(D_Width-1):='0';   -- msb
         when ADD => RVar := signed('0' & RdVar) +  signed('0' & RsVar);
                     RdOutVar := std_logic_vector(RVar(D_Width-1 downto 0));
                     NFlag <= RVar(D_Width);
         when SUB => RVar := signed('0' & RdVar) -  signed('0' & RsVar);
                     RdOutVar := std_logic_vector(RVar(D_Width-1 downto 0));
                     NFlag <= RVar(D_Width);
         when ANDA=> RdOutVar := RdVar AND RsVar;
         when EOR => RdOutVar := RdVar XOR RsVar;
         when ORA => RdOutVar := RdVar OR RsVar;
         when STR | PUSH => DS_DI <= Akku;
         when LDA | POP  => RdOutVar := DS_DO;
         when LDI => RdOutVar := K;
         
         when  others => null; -- no action 
      end case;     
      if (RdOutVar = Zero) then ZFlag <='1'; end if;
      RdOut <= RdOutVar;
  end process;
  
  load_akku : process (Clk) begin
   if (ACC_EN = '1') then
      if rising_edge(Clk) then
          Akku <= RdOut;
      end if;
    end if;
  end process load_akku;

end RTL;